/*
    Copyright (c) 2002-2009 Tampere University of Technology.

    This file is part of TTA-Based Codesign Environment (TCE).

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
 */
/**
 * @file Address.hh
 *
 * Declaration of Address class.
 *
 * @author Ari Metshalme 2005 (ari.metsahalme-no.spam-tut.fi)
 * @note rating: red
 */

#ifndef TTA_ADDRESS_HH
#define TTA_ADDRESS_HH

#include "BaseType.hh"

namespace TTAMachine {
    class AddressSpace;
}

namespace TTAProgram {

/**
 * An address in the target memory system of the TTA program.
 *
 * An address is a number that identifies a location in an address space. An
 * address can point to a location of data memory, instruction memory or
 * unified memory.
 */
class Address {
public:
    Address(
        InstructionAddress location,
        const TTAMachine::AddressSpace& space);
    ~Address();

    InstructionAddress location() const;
    const TTAMachine::AddressSpace& space() const;

private:
    /// The location identified by the address.
    InstructionAddress location_;
    /// The address space of the address.
    const TTAMachine::AddressSpace* space_;
};

#include "Address.icc"

}

#endif
